VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsTrickFFT"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' clsTrickFFT  -     
'     (The trick), 2014

Option Explicit

Public Enum WindowType
    WT_RECTANGLE
    WT_TRIGANULAR
    WT_HAMMING
    WT_HANN
End Enum

Private Coef(1, 13) As Single
Private mFFTSize    As Long
Private mLog        As Long
Private mWindow()   As Single
Private mType       As WindowType

' //  
Public Property Get WindowType() As WindowType
    WindowType = mType
End Property
Public Property Let WindowType(ByVal Value As WindowType)

    If InitWindow(Value) Then
    
        mType = Value
        
    End If
    
End Property

' //   FFT
Public Property Let FFTSize(ByVal Value As Long)
    Dim log2    As Double
    
    log2 = Log(Value) / Log(2)
    '     2-
    If log2 <> Fix(log2) Then
        err.Raise 5
        Exit Property
    End If
    '    
    If log2 < 2 Or log2 > 16384 Then
        err.Raise 9
        Exit Property
    End If
    
    InitWindow mType
    
    mLog = log2
    mFFTSize = Value
    
End Property

' //   
Public Function ApplyWindow(data() As Single) As Boolean
    Dim index   As Long
    Dim count   As Long
    
    count = UBound(data, 2) + 1

    For index = 0 To count - 1
        data(0, index) = data(0, index) * mWindow(index)
    Next
    
    ApplyWindow = True
    
End Function

' //  16-     
Public Function Convert16BitToComplex(inData() As Integer, outData() As Single) As Boolean
    Dim index   As Long

    For index = 0 To UBound(inData)
        outData(0, index) = inData(index) / 32768
        outData(1, index) = 0
    Next
    
    Convert16BitToComplex = True
    
End Function

' //   ,     16- 
Public Function ConvertComplexTo16Bit(inData() As Single, outData() As Integer) As Boolean
    Dim index   As Long
    Dim Value   As Long
    
    For index = 0 To UBound(inData, 2)
        Value = inData(0, index) * 32767
        If Value > 32767 Then Value = 32767 Else If Value < -32768 Then Value = -32768
        outData(index) = Value
    Next
    
    ConvertComplexTo16Bit = True
        
End Function

' //  
Public Function MakeMirror(data() As Single) As Boolean
    Dim index   As Long
    Dim pointer As Long
    
    pointer = mFFTSize - 1
    
    For index = 1 To mFFTSize \ 2 - 1
        data(0, pointer) = data(0, index)
        data(1, pointer) = -data(1, index)
        pointer = pointer - 1
    Next
    
    MakeMirror = True
    
End Function

' //   
Public Function FFT(data() As Single, ByVal IsInverse As Boolean) As Boolean
    Dim i As Long, j As Long, n As Long, K As Long, io As Long, ie As Long, in_ As Long, nn As Long
    Dim ur As Single, ui As Single, tpr As Single, tpi As Single, tqr As Single, tqi As Single, _
        wr As Single, wi As Single, sr As Single, ti As Long, tr As Long
    
    nn = mFFTSize \ 2: ie = mFFTSize
    For n = 1 To mLog
        wr = Coef(0, mLog - n): wi = Coef(1, mLog - n)
        If IsInverse Then wi = -wi
        in_ = ie \ 2: ur = 1: ui = 0
        For j = 0 To in_ - 1
            For i = j To mFFTSize - 1 Step ie
                io = i + in_
                tpr = data(0, i) + data(0, io): tpi = data(1, i) + data(1, io)
                tqr = data(0, i) - data(0, io): tqi = data(1, i) - data(1, io)
                data(0, io) = tqr * ur - tqi * ui: data(1, io) = tqi * ur + tqr * ui
                data(0, i) = tpr: data(1, i) = tpi
            Next
            sr = ur: ur = ur * wr - ui * wi: ui = ui * wr + sr * wi
        Next
        ie = ie \ 2
    Next
    ' 
    j = 1
    For i = 1 To mFFTSize - 1
        If i < j Then
            io = i - 1: in_ = j - 1: tpr = data(0, in_): tpi = data(1, in_)
            data(0, in_) = data(0, io): data(1, in_) = data(1, io)
            data(0, io) = tpr: data(1, io) = tpi
        End If
        K = nn
        Do While K < j
            j = j - K: K = K \ 2
        Loop
        j = j + K
    Next
    If IsInverse Then FFT = True: Exit Function
    ' 
    wr = 1 / mFFTSize
    For i = 0 To mFFTSize - 1
        data(0, i) = data(0, i) * wr: data(1, i) = data(1, i) * wr
    Next
    FFT = True
    
End Function

' //  
Public Function InitWindow(ByVal Window As WindowType) As Boolean
    Dim index   As Long
    
    Select Case Window
    Case WT_RECTANGLE
        ReDim mWindow(mFFTSize - 1)
        For index = 0 To mFFTSize - 1
            mWindow(index) = 1
        Next
    Case WT_TRIGANULAR
        ReDim mWindow(mFFTSize - 1)
        For index = 0 To mFFTSize - 1
            mWindow(index) = IIf(index < mFFTSize \ 2, index / mFFTSize * 2, 1 - index / (mFFTSize - 1))
        Next
    Case WT_HAMMING
        ReDim mWindow(mFFTSize - 1)
        For index = 0 To mFFTSize - 1
            mWindow(index) = 0.53836 - 0.46164 * Cos(6.28318530717959 * index / (mFFTSize - 1))
        Next
    Case WT_HANN
        ReDim mWindow(mFFTSize - 1)
        For index = 0 To mFFTSize - 1
            mWindow(index) = 0.5 * (1 - Cos(6.28318530717959 * index / (mFFTSize - 1)))
        Next
    Case Else
        err.Raise 5
        Exit Function
    End Select

    InitWindow = True
    
End Function

' //     FFT    
Private Sub Class_Initialize()
    Dim n As Long, vRcoef As Variant, vIcoef As Variant
    vRcoef = Array(-1#, 0#, 0.707106781186547 _
          , 0.923879532511287, 0.98078528040323, 0.995184726672197 _
          , 0.998795456205172, 0.999698818696204, 0.999924701839145 _
          , 0.999981175282601, 0.999995293809576, 0.999998823451702 _
          , 0.999999705862882, 0.999999926465718)
    vIcoef = Array(0#, -1#, -0.707106781186547 _
         , -0.38268343236509, -0.195090322016128, -9.80171403295606E-02 _
         , -0.049067674327418, -2.45412285229122E-02, -1.22715382857199E-02 _
         , -6.1358846491544E-03, -3.0679567629659E-03, -1.5339801862847E-03 _
         , -7.669903187427E-04, -3.834951875714E-04)
    For n = 0 To 13
        Coef(0, n) = vRcoef(n): Coef(1, n) = vIcoef(n)
    Next
    
    mFFTSize = 512
    mLog = 9
    mType = WT_HAMMING
    InitWindow mType
    
End Sub


