VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsTrickWavConverter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' clsTrickWavConverter -    Wav   ACM
'     (The trick), 2014

Option Explicit

Private Type WAVEFORMATEX
    wFormatTag      As Integer
    nChannels       As Integer
    nSamplesPerSec  As Long
    nAvgBytesPerSec As Long
    nBlockAlign     As Integer
    wBitsPerSample  As Integer
    cbSize          As Integer
End Type

Private Type ACMSTREAMHEADER
    cbStruct        As Long
    fdwStatus       As Long
    lpdwUser        As Long
    lppbSrc         As Long
    cbSrcLength     As Long
    cbSrcLengthUsed As Long
    lpdwSrcUser     As Long
    lppbDst         As Long
    cbDstLength     As Long
    cbDstLengthUsed As Long
    lpdwDstUser     As Long
    dwDriver(9)     As Long
End Type

Private Type MMCKINFO
    ckid            As Long
    ckSize          As Long
    fccType         As Long
    dwDataOffset    As Long
    dwFlags         As Long
End Type

Private Declare Function acmStreamClose Lib "msacm32" (ByVal has As Long, ByVal fdwClose As Long) As Long
Private Declare Function acmStreamConvert Lib "msacm32" (ByVal has As Long, ByRef pash As ACMSTREAMHEADER, ByVal fdwConvert As Long) As Long
Private Declare Function acmStreamMessage Lib "msacm32" (ByVal has As Long, ByVal uMsg As Long, ByVal lParam1 As Long, ByVal lParam2 As Long) As Long
Private Declare Function acmStreamOpen Lib "msacm32" (phas As Any, ByVal had As Long, pwfxSrc As WAVEFORMATEX, pwfxDst As WAVEFORMATEX, pwfltr As Any, dwCallback As Any, dwInstance As Any, ByVal fdwOpen As Long) As Long
Private Declare Function acmStreamPrepareHeader Lib "msacm32" (ByVal has As Long, ByRef pash As ACMSTREAMHEADER, ByVal fdwPrepare As Long) As Long
Private Declare Function acmStreamReset Lib "msacm32" (ByVal has As Long, ByVal fdwReset As Long) As Long
Private Declare Function acmStreamSize Lib "msacm32" (ByVal has As Long, ByVal cbInput As Long, ByRef pdwOutputBytes As Long, ByVal fdwSize As Long) As Long
Private Declare Function acmStreamUnprepareHeader Lib "msacm32" (ByVal has As Long, ByRef pash As ACMSTREAMHEADER, ByVal fdwUnprepare As Long) As Long

Private Declare Function mmioClose Lib "winmm.dll" (ByVal hmmio As Long, ByVal uFlags As Long) As Long
Private Declare Function mmioDescend Lib "winmm.dll" (ByVal hmmio As Long, lpck As MMCKINFO, lpckParent As Any, ByVal uFlags As Long) As Long
Private Declare Function mmioOpen Lib "winmm.dll" Alias "mmioOpenA" (ByVal szFileName As String, lpmmioinfo As Any, ByVal dwOpenFlags As Long) As Long
Private Declare Function mmioRead Lib "winmm.dll" (ByVal hmmio As Long, pch As Any, ByVal cch As Long) As Long
Private Declare Function mmioStringToFOURCC Lib "winmm.dll" Alias "mmioStringToFOURCCA" (ByVal sz As String, ByVal uFlags As Long) As Long
Private Declare Function mmioAscend Lib "winmm.dll" (ByVal hmmio As Long, lpck As MMCKINFO, ByVal uFlags As Long) As Long

Private Const MMIO_READ                     As Long = &H0
Private Const MMIO_FINDCHUNK                As Long = &H10
Private Const MMIO_FINDRIFF                 As Long = &H20
Private Const ACM_STREAMOPENF_QUERY         As Long = &H1
Private Const ACM_STREAMSIZEF_DESTINATION   As Long = &H1&
Private Const ACM_STREAMSIZEF_SOURCE        As Long = &H0&
Private Const ACM_STREAMCONVERTF_BLOCKALIGN As Long = &H4
Private Const ACM_STREAMCONVERTF_START      As Long = &H10

Private mInpFmt     As WAVEFORMATEX     '  ,  
Private mOutFmt     As WAVEFORMATEX     '  ,  
Private mDataSize   As Long             '    
Private bufIdx      As Long             '     
Private buffer()    As Byte             ' 
Private hStream     As Long             '   
Private mInit       As Boolean          '   ACM

' //  
Public Property Get InputNumOfChannels() As Integer
    InputNumOfChannels = mInpFmt.nChannels
End Property
Public Property Get InputSamplesPerSecond() As Integer
    InputSamplesPerSecond = mInpFmt.nSamplesPerSec
End Property
Public Property Get InputBitPerSample() As Integer
    InputBitPerSample = mInpFmt.wBitsPerSample
End Property

' //   
Public Property Get InputDataSize() As Long
    InputDataSize = mDataSize
End Property

' //      
Public Property Get InputCurrentPosition() As Long
    InputCurrentPosition = bufIdx / mInpFmt.nBlockAlign
End Property
Public Property Let InputCurrentPosition(ByVal Value As Long)
    Dim index As Long
    
    index = Value * mInpFmt.nBlockAlign
    
    If index >= mDataSize Or index < 0 Then
        
        err.Raise 5
        Exit Property
        
    End If
    
    bufIdx = index
End Property

' //  
Public Property Get OutputNumOfChannels() As Integer
    OutputNumOfChannels = mOutFmt.nChannels
End Property
Public Property Get OutputSamplesPerSecond() As Integer
    OutputSamplesPerSecond = mOutFmt.nSamplesPerSec
End Property
Public Property Get OutputBitPerSample() As Integer
    OutputBitPerSample = mOutFmt.wBitsPerSample
End Property

' //  
Public Property Get Rate() As Single
    Dim outLen  As Long
    '   
    If Not mInit Then
        If Not Init() Then Exit Property
    End If
    acmStreamSize hStream, mDataSize, outLen, ACM_STREAMSIZEF_SOURCE
    Rate = outLen / mDataSize
End Property

' //  
Public Function SetFormat(ByVal NumOfChannels As Integer, ByVal SamplesPerSecond As Long, ByVal BitPerSample As Integer) As Boolean
    Dim outFmt  As WAVEFORMATEX
    Dim ret     As Long
    '  
    With outFmt
        .wFormatTag = 1
        .nChannels = NumOfChannels
        .nSamplesPerSec = SamplesPerSecond
        .wBitsPerSample = BitPerSample
        .nBlockAlign = .wBitsPerSample \ 8 * .nChannels
        .nAvgBytesPerSec = .nSamplesPerSec * .nBlockAlign
    End With
    '   
    If mDataSize Then
        '    ,         
        ret = acmStreamOpen(ByVal 0&, 0, mInpFmt, outFmt, ByVal 0&, ByVal 0&, ByVal 0&, ACM_STREAMOPENF_QUERY)
        If ret Then Exit Function
        '   
        If hStream Then acmStreamClose hStream, 0
        mInit = False
    End If

    mOutFmt = outFmt
    SetFormat = True
    
End Function

' //  Wav        
Public Function ReadWaveFile(strFileName As String) As Boolean
    Dim hIn     As Long
    Dim inf     As MMCKINFO
    Dim sInf    As MMCKINFO
    Dim inpFmt  As WAVEFORMATEX
    Dim ret     As Long
    '  
    hIn = mmioOpen(strFileName, ByVal 0, MMIO_READ)
    If (hIn = 0) Then
        MsgBox "Error opening file"
        Exit Function
    End If
    '   WAVE
    inf.fccType = mmioStringToFOURCC("WAVE", 0)
    If mmioDescend(hIn, inf, ByVal 0, MMIO_FINDRIFF) Then
        mmioClose hIn, 0
        MsgBox "Is not valid file"
        Exit Function
    End If
    '   fmt,   
    sInf.ckid = mmioStringToFOURCC("fmt", 0)
    If mmioDescend(hIn, sInf, inf, MMIO_FINDCHUNK) Then
        mmioClose hIn, 0
        MsgBox "Format chunk not found"
        Exit Function
    End If
    '  
    If sInf.ckSize > Len(inpFmt) Then
        mmioClose hIn, 0
        MsgBox "Not supported format"
        Exit Function
    End If
    '  
    If mmioRead(hIn, inpFmt, sInf.ckSize) = -1 Then
        mmioClose hIn, 0
        MsgBox "Can't read format"
        Exit Function
    End If
    '    ,         
    ret = acmStreamOpen(ByVal 0&, 0, inpFmt, mOutFmt, ByVal 0&, ByVal 0&, ByVal 0&, ACM_STREAMOPENF_QUERY)
    If ret Then
        mmioClose hIn, 0
        MsgBox "Can't convert wav file"
        Exit Function
    End If
    '    fmt
    mmioAscend hIn, sInf, 0
    '   data  
    sInf.ckid = mmioStringToFOURCC("data", 0)
    If mmioDescend(hIn, sInf, inf, MMIO_FINDCHUNK) Then
        mmioClose hIn, 0
        MsgBox "Wave data not found"
        Exit Function
    End If
    '  
    If sInf.ckSize <= 0 Then
        mmioClose hIn, 0
        MsgBox "Invalid data size"
        Exit Function
    End If
    '     
    ReDim buffer(sInf.ckSize - 1)
    If mmioRead(hIn, buffer(0), sInf.ckSize) = -1 Then
        mmioClose hIn, 0
        MsgBox "Can't read data"
        Exit Function
    End If
    '  
    mmioClose hIn, 0
    '  
    mDataSize = sInf.ckSize
    bufIdx = 0
    mInpFmt = inpFmt
    ReadWaveFile = True
    
End Function

' //   
Public Function Convert(ByVal lpOutData As Long, ByVal dwCountBytes As Long, dwCountRead As Long) As Boolean
    Dim ret             As Long
    Dim inpCountBytes   As Long
    Dim acmHdr          As ACMSTREAMHEADER
    '   
    If Not mInit Then
        If Not Init() Then Exit Function
    End If
    '          
    ret = acmStreamSize(hStream, dwCountBytes, inpCountBytes, ACM_STREAMSIZEF_DESTINATION)
    If ret Then Exit Function
    '       
    If inpCountBytes + bufIdx >= mDataSize Then
        inpCountBytes = mDataSize - bufIdx
        
        If inpCountBytes <= 0 Then
            Convert = True
            dwCountRead = 0
            Exit Function
        End If
        
    End If
    '   
    With acmHdr
        .cbStruct = Len(acmHdr)
        .lppbDst = lpOutData
        .lppbSrc = VarPtr(buffer(bufIdx))
        .cbDstLength = dwCountBytes
        .cbSrcLength = inpCountBytes
    End With
    '   
    ret = acmStreamPrepareHeader(hStream, acmHdr, 0)
    If ret Then Exit Function
    ' 
    ret = acmStreamConvert(hStream, acmHdr, ACM_STREAMCONVERTF_BLOCKALIGN)
    ' 
    acmStreamUnprepareHeader hStream, acmHdr, 0
    If ret Then Exit Function
    '     
    dwCountRead = acmHdr.cbDstLengthUsed
    bufIdx = bufIdx + acmHdr.cbSrcLengthUsed
    ' 
    Convert = True
    
End Function

' //   ACM
Private Function Init() As Boolean
    Dim ret As Long
    '     
    ret = acmStreamOpen(hStream, 0, mInpFmt, mOutFmt, ByVal 0&, ByVal 0&, ByVal 0&, 0)
    If ret Then Exit Function
    
    Init = True
    mInit = True
End Function

Private Sub Class_Initialize()
    '    
    With mOutFmt
        .wFormatTag = 1
        .nChannels = 1
        .nSamplesPerSec = SampleRate
        .wBitsPerSample = 16
        .nBlockAlign = .wBitsPerSample \ 8 * .nChannels
        .nAvgBytesPerSec = .nSamplesPerSec * .nBlockAlign
    End With
End Sub

Private Sub Class_Terminate()
    If hStream Then acmStreamClose hStream, 0
End Sub
